#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.
#

from struct import pack, unpack
from talos.thrift.Thrift import TException
from ..compat import BufferIO


class TTransportException(TException):
    """Custom Transport Exception class"""

    UNKNOWN = 0
    NOT_OPEN = 1
    ALREADY_OPEN = 2
    TIMED_OUT = 3
    END_OF_FILE = 4
    NEGATIVE_SIZE = 5
    SIZE_LIMIT = 6
    INVALID_CLIENT_TYPE = 7

    def __init__(self, type=UNKNOWN, message=None, inner=None):
        TException.__init__(self, message)
        self.type = type
        self.inner = inner


class TTransportBase(object):
    """Base class for Thrift transport layer."""

    def isOpen(self):
        pass

    def open(self):
        pass

    def close(self):
        pass

    def read(self, sz):
        pass

    def readAll(self, sz):
        buff = b''
        have = 0
        while (have < sz):
            chunk = self.read(sz - have)
            chunkLen = len(chunk)
            have += chunkLen
            buff += chunk

            if chunkLen == 0:
                raise EOFError()

        return buff

    def write(self, buf):
        pass

    def flush(self):
        pass


# This class should be thought of as an interface.
class CReadableTransport(object):
    """base class for transports that are readable from C"""

    # TODO(dreiss): Think about changing this interface to allow us to use
    #               a (Python, not c) StringIO instead, because it allows
    #               you to write after reading.

    # NOTE: This is a classic class, so properties will NOT work
    #       correctly for setting.
    @property
    def cstringio_buf(self):
        """A cStringIO buffer that contains the current chunk we are reading."""
        pass

    def cstringio_refill(self, partialread, reqlen):
        """Refills cstringio_buf.

        Returns the currently used buffer (which can but need not be the same as
        the old cstringio_buf). partialread is what the C code has read from the
        buffer, and should be inserted into the buffer before any more reads.  The
        return value must be a new, not borrowed reference.  Something along the
        lines of self._buf should be fine.

        If reqlen bytes can't be read, throw EOFError.
        """
        pass


class TServerTransportBase(object):
    """Base class for Thrift server transports."""

    def listen(self):
        pass

    def accept(self):
        pass

    def close(self):
        pass


class TTransportFactoryBase(object):
    """Base class for a Transport Factory"""

    def getTransport(self, trans):
        return trans


class TBufferedTransportFactory(object):
    """Factory transport that builds buffered transports"""

    def getTransport(self, trans):
        buffered = TBufferedTransport(trans)
        return buffered


class TBufferedTransport(TTransportBase, CReadableTransport):
    """Class that wraps another transport and buffers its I/O.

    The implementation uses a (configurable) fixed-size read buffer
    but buffers all writes until a flush is performed.
    """
    DEFAULT_BUFFER = 4096

    def __init__(self, trans, rbuf_size=DEFAULT_BUFFER):
        self.__trans = trans
        self.__wbuf = BufferIO()
        # Pass string argument to initialize read buffer as cStringIO.InputType
        self.__rbuf = BufferIO(b'')
        self.__rbuf_size = rbuf_size

    def isOpen(self):
        return self.__trans.isOpen()

    def open(self):
        return self.__trans.open()

    def close(self):
        return self.__trans.close()

    def read(self, sz):
        ret = self.__rbuf.read(sz)
        if len(ret) != 0:
            return ret
        self.__rbuf = BufferIO(self.__trans.read(max(sz, self.__rbuf_size)))
        return self.__rbuf.read(sz)

    def write(self, buf):
        try:
            self.__wbuf.write(buf)
        except Exception as e:
            # on exception reset wbuf so it doesn't contain a partial function call
            self.__wbuf = BufferIO()
            raise e

    def flush(self):
        out = self.__wbuf.getvalue()
        # reset wbuf before write/flush to preserve state on underlying failure
        self.__wbuf = BufferIO()
        self.__trans.write(out)
        self.__trans.flush()

    # Implement the CReadableTransport interface.
    @property
    def cstringio_buf(self):
        return self.__rbuf

    def cstringio_refill(self, partialread, reqlen):
        retstring = partialread
        if reqlen < self.__rbuf_size:
            # try to make a read of as much as we can.
            retstring += self.__trans.read(self.__rbuf_size)

        # but make sure we do read reqlen bytes.
        if len(retstring) < reqlen:
            retstring += self.__trans.readAll(reqlen - len(retstring))

        self.__rbuf = BufferIO(retstring)
        return self.__rbuf


class TMemoryBuffer(TTransportBase, CReadableTransport):
    """Wraps a cBytesIO object as a TTransport.

    NOTE: Unlike the C++ version of this class, you cannot write to it
          then immediately read from it.  If you want to read from a
          TMemoryBuffer, you must either pass a string to the constructor.
    TODO(dreiss): Make this work like the C++ version.
    """

    def __init__(self, value=None, offset=0):
        """value -- a value to read from for stringio

        If value is set, this will be a transport for reading,
        otherwise, it is for writing"""
        if value is not None:
            self._buffer = BufferIO(value)
        else:
            self._buffer = BufferIO()
        if offset:
            self._buffer.seek(offset)

    def isOpen(self):
        return not self._buffer.closed

    def open(self):
        pass

    def close(self):
        self._buffer.close()

    def read(self, sz):
        return self._buffer.read(sz)

    def write(self, buf):
        self._buffer.write(buf)

    def flush(self):
        pass

    def getvalue(self):
        return self._buffer.getvalue()

    # Implement the CReadableTransport interface.
    @property
    def cstringio_buf(self):
        return self._buffer

    def cstringio_refill(self, partialread, reqlen):
        # only one shot at reading...
        raise EOFError()


class TFramedTransportFactory(object):
    """Factory transport that builds framed transports"""

    def getTransport(self, trans):
        framed = TFramedTransport(trans)
        return framed


class TFramedTransport(TTransportBase, CReadableTransport):
    """Class that wraps another transport and frames its I/O when writing."""

    def __init__(self, trans,):
        self.__trans = trans
        self.__rbuf = BufferIO(b'')
        self.__wbuf = BufferIO()

    def isOpen(self):
        return self.__trans.isOpen()

    def open(self):
        return self.__trans.open()

    def close(self):
        return self.__trans.close()

    def read(self, sz):
        ret = self.__rbuf.read(sz)
        if len(ret) != 0:
            return ret

        self.readFrame()
        return self.__rbuf.read(sz)

    def readFrame(self):
        buff = self.__trans.readAll(4)
        sz, = unpack('!i', buff)
        self.__rbuf = BufferIO(self.__trans.readAll(sz))

    def write(self, buf):
        self.__wbuf.write(buf)

    def flush(self):
        wout = self.__wbuf.getvalue()
        wsz = len(wout)
        # reset wbuf before write/flush to preserve state on underlying failure
        self.__wbuf = BufferIO()
        # N.B.: Doing this string concatenation is WAY cheaper than making
        # two separate calls to the underlying socket object. Socket writes in
        # Python turn out to be REALLY expensive, but it seems to do a pretty
        # good job of managing string buffer operations without excessive copies
        buf = pack("!i", wsz) + wout
        self.__trans.write(buf)
        self.__trans.flush()

    # Implement the CReadableTransport interface.
    @property
    def cstringio_buf(self):
        return self.__rbuf

    def cstringio_refill(self, prefix, reqlen):
        # self.__rbuf will already be empty here because fastbinary doesn't
        # ask for a refill until the previous buffer is empty.  Therefore,
        # we can start reading new frames immediately.
        while len(prefix) < reqlen:
            self.readFrame()
            prefix += self.__rbuf.getvalue()
        self.__rbuf = BufferIO(prefix)
        return self.__rbuf


class TFileObjectTransport(TTransportBase):
    """Wraps a file-like object to make it work as a Thrift transport."""

    def __init__(self, fileobj):
        self.fileobj = fileobj

    def isOpen(self):
        return True

    def close(self):
        self.fileobj.close()

    def read(self, sz):
        return self.fileobj.read(sz)

    def write(self, buf):
        self.fileobj.write(buf)

    def flush(self):
        self.fileobj.flush()


class TSaslClientTransport(TTransportBase, CReadableTransport):
    """
    SASL transport
    """

    START = 1
    OK = 2
    BAD = 3
    ERROR = 4
    COMPLETE = 5

    def __init__(self, transport, host, service, mechanism='GSSAPI',
                 **sasl_kwargs):
        """
        transport: an underlying transport to use, typically just a TSocket
        host: the name of the server, from a SASL perspective
        service: the name of the server's service, from a SASL perspective
        mechanism: the name of the preferred mechanism to use

        All other kwargs will be passed to the puresasl.client.SASLClient
        constructor.
        """

        from puresasl.client import SASLClient

        self.transport = transport
        self.sasl = SASLClient(host, service, mechanism, **sasl_kwargs)

        self.__wbuf = BufferIO()
        self.__rbuf = BufferIO(b'')

    def open(self):
        if not self.transport.isOpen():
            self.transport.open()

        self.send_sasl_msg(self.START, bytes(self.sasl.mechanism, 'ascii'))
        self.send_sasl_msg(self.OK, self.sasl.process())

        while True:
            status, challenge = self.recv_sasl_msg()
            if status == self.OK:
                self.send_sasl_msg(self.OK, self.sasl.process(challenge))
            elif status == self.COMPLETE:
                if not self.sasl.complete:
                    raise TTransportException(
                        TTransportException.NOT_OPEN,
                        "The server erroneously indicated "
                        "that SASL negotiation was complete")
                else:
                    break
            else:
                raise TTransportException(
                    TTransportException.NOT_OPEN,
                    "Bad SASL negotiation status: %d (%s)"
                    % (status, challenge))

    def send_sasl_msg(self, status, body):
        header = pack(">BI", status, len(body))
        self.transport.write(header + body)
        self.transport.flush()

    def recv_sasl_msg(self):
        header = self.transport.readAll(5)
        status, length = unpack(">BI", header)
        if length > 0:
            payload = self.transport.readAll(length)
        else:
            payload = ""
        return status, payload

    def write(self, data):
        self.__wbuf.write(data)

    def flush(self):
        data = self.__wbuf.getvalue()
        encoded = self.sasl.wrap(data)
        self.transport.write(pack("!i", len(encoded)) + encoded)
        self.transport.flush()
        self.__wbuf = BufferIO()

    def read(self, sz):
        ret = self.__rbuf.read(sz)
        if len(ret) != 0:
            return ret

        self._read_frame()
        return self.__rbuf.read(sz)

    def _read_frame(self):
        header = self.transport.readAll(4)
        length, = unpack('!i', header)
        encoded = self.transport.readAll(length)
        self.__rbuf = BufferIO(self.sasl.unwrap(encoded))

    def close(self):
        self.sasl.dispose()
        self.transport.close()

    # based on TFramedTransport
    @property
    def cstringio_buf(self):
        return self.__rbuf

    def cstringio_refill(self, prefix, reqlen):
        # self.__rbuf will already be empty here because fastbinary doesn't
        # ask for a refill until the previous buffer is empty.  Therefore,
        # we can start reading new frames immediately.
        while len(prefix) < reqlen:
            self._read_frame()
            prefix += self.__rbuf.getvalue()
        self.__rbuf = BufferIO(prefix)
        return self.__rbuf



#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 

from talos.thrift.common.ttypes import GalaxyTalosException
from talos.thrift.topic.ttypes import TopicAndPartition
from talos.producer.UserMessageResult import UserMessageResult
from talos.producer.SimpleProducer import SimpleProducer
from talos.utils.Utils import synchronized
from talos.utils import Utils
from talos.producer.PartitionMessageQueue import PartitionMessageQueue
from concurrent.futures import ThreadPoolExecutor
import logging
import threading
import traceback


class TASK_STATE:
    ACTIVE = 0
    CLOSE = 1


class MessageWriter(threading.Thread):

    def __init__(self, partitionSender=None, globalLock=None):
        threading.Thread.__init__(self, name="messageWriter-" +
                                             partitionSender.topicAndPartition.topicName
                                             + "-" + str(partitionSender.topicAndPartition.partitionId))
        self.partitionSender = partitionSender
        self.globalLock = globalLock
        self.simpleProducer = SimpleProducer(
            producerConfig=partitionSender.talosProducerConfig,
            topicAndPartition=partitionSender.topicAndPartition,
            messageClient=partitionSender.messageClient,
            clientId=partitionSender.clientId,
            requestId=partitionSender.requestId)
        self.synchronizedLock = threading.Lock()

    def run(self):
        while self.partitionSender.get_cur_state() == TASK_STATE.ACTIVE:
            try:
                messageList = self.partitionSender.partitionMessageQueue.get_message_list()

                # when messageList return no message, this means TalosProducer not
                # alive and there is no more message to send, then we should exit
                # write message right now;
                if len(messageList) == 0:
                    # notify to wake up producer's global lock
                    with self.synchronizedLock:
                        if self.globalLock.acquire():
                            self.globalLock.notifyAll()
                            self.globalLock.release()
                    break

                self.partitionSender.put_message(messageList, self.simpleProducer)
            except Exception as e:
                if isinstance(e, GalaxyTalosException):
                    self.partitionSender.logger.error(e.errMsg)
                self.partitionSender.logger.error(
                    "PutMessageTask for topicAndPartition: " +
                    str(self.partitionSender.topicAndPartition)
                    + " failed" + str(traceback.format_exc()))
            finally:
                # notify to wake up producer's global lock
                with self.synchronizedLock:
                    if self.globalLock.acquire():
                        self.globalLock.notifyAll()
                        self.globalLock.release()

        self.partitionSender.process_buffered_message()


class MessageCallbackTask(threading.Thread):
    def __init__(self, userMessageResult=None, userMessageCallback=None):
        threading.Thread.__init__(self)
        self.userMessageCallback = userMessageCallback
        self.userMessageResult = userMessageResult

    def run(self):
        if self.userMessageResult.is_successful():
            self.userMessageCallback.on_success(self.userMessageResult)
        else:
            self.userMessageCallback.on_error(self.userMessageResult)


class PartitionSender:
    logger = logging.getLogger("PartitionSender")

    def __init__(self, partitionId=None, topicName=None, topicTalosResourceName=None,
                 requestId=None, clientId=None, talosProducerConfig=None,
                 messageClient=None, userMessageCallback=None, messageCallbackExecutors=None,
                 globalLock=None, producer=None):
        self.partitionId = partitionId
        # a static attribute of TalosProducer which will guarantee that
        # the requestId of all PartitionSender always be global unique
        self.requestId = requestId
        self.clientId = clientId
        self.talosProducerConfig = talosProducerConfig
        self.messageClient = messageClient
        self.userMessageCallback = userMessageCallback
        self.messageCallbackExecutors = messageCallbackExecutors
        self.globalLock = globalLock
        self.producer = producer
        self.synchronizedLock = threading.Lock()

        self.curState = TASK_STATE.ACTIVE
        self.topicAndPartition = TopicAndPartition(topicName,
                                                   topicTalosResourceName,
                                                   partitionId)
        self.partitionMessageQueue = PartitionMessageQueue(talosProducerConfig,
                                                           partitionId, producer)
        self.singleExecutor = ThreadPoolExecutor(
            max_workers=1, thread_name_prefix="talos-producer-" +
                                              self.topicAndPartition.topicName
                                              + " " + str(self.topicAndPartition.partitionId))

        self.executor = MessageWriter(self, self.globalLock)
        self.executor.start()

    def put_message(self, messageList=None, simpleProducer=None):
        userMessageResult = UserMessageResult(messageList,
                                              self.topicAndPartition.topicName,
                                              self.partitionId)
        try:
            # when TalosProducer is disabled, we just fail the message and inform user;
            # but when TalosProducer is shutdown, we will send the left message.
            if self.producer.is_disabled():
                e = Exception()
                e.message = "The Topic: " + self.topicAndPartition.topicName + \
                            " with resourceName: " + self.topicAndPartition.topicTalosResourceName.topicTalosResourceName \
                            + " no longer exist. Please check the topic and" +  \
                            " reconstruct the TalosProducer again"
                raise e
            else:
                simpleProducer.do_put(messageList)
                # putMessage success callback
                userMessageResult.set_successful(True)
                self.messageCallbackExecutors = MessageCallbackTask(userMessageResult,
                                                                    self.userMessageCallback)
                self.messageCallbackExecutors.start()
                if self.logger.isEnabledFor(logging.DEBUG):
                    self.logger.debug("put " + messageList.size() +
                                      " message success for partition: "
                                      + str(self.partitionId))
        except Exception as e:
            # 1. backoff sleep and retry according to configuration
            # 2. clear partition buffer when long time failed and
            # queue size greater than threshold
            # 3. truly failed, execute callback task
            self.process_fetched_exception(e, userMessageResult, simpleProducer)

    def process_fetched_exception(self, e=None, userMessageResult=None, simpleProducer=None):
        retrySuccess = self.retry_put_message(e, userMessageResult, simpleProducer)
        if not retrySuccess:
            # execute putMessage failed callback when all retry failed
            userMessageResult.set_successful(False)
            userMessageResult.set_cause(e)
            self.messageCallbackExecutors = MessageCallbackTask(userMessageResult,
                                                                self.userMessageCallback)
            self.messageCallbackExecutors.start()
            if self.logger.isEnabledFor(logging.DEBUG):
                for message in userMessageResult.get_message_list():
                    self.logger.error(message.sequenceNumber + ": " + str(message.message))

            if not Utils.is_partition_not_serving(e):
                if isinstance(e, GalaxyTalosException):
                    self.logger.error(e.errMsg)
                self.logger.error("Failed to put " + str(len(userMessageResult.get_message_list())) +
                                  " messages for partition: " + str(self.partitionId)
                                  + str(traceback.format_exc()) )

    def retry_put_message(self, e=None, userMessageResult=None, simpleProducer=None):
        if self.talosProducerConfig.get_max_retry() <= 0:
            return False

        retry = 0
        while retry <= self.talosProducerConfig.get_max_retry():
            pauseTime = Utils.get_put_msg_failed_delay(retry, self.talosProducerConfig)
            retry += 1
            if self.logger.isEnabledFor(logging.DEBUG):
                self.logger.warn("putMessage failed " + str(retry) +
                                 " times for partition: " + str(self.partitionId) +
                                 ", sleep " + pauseTime +
                                 "ms for avoid infinitely retry." + e.message)
            # 1. sleep
            Utils.sleep_pause_time(pauseTime)

            # 2. retry putMessage, just return if retry success
            try:
                simpleProducer.do_put(userMessageResult.get_message_list())
                userMessageResult.set_successful(True)
                self.messageCallbackExecutors = MessageCallbackTask(userMessageResult,
                                                                    self.userMessageCallback)
                self.messageCallbackExecutors.start()
                return True
            except Exception as ex:
                if isinstance(ex, GalaxyTalosException):
                    self.logger.error(ex.errMsg)
                self.logger.error("retry put message failed. " + str(traceback.format_exc()))

            # 3. clear messageQueue when retry > 10 and partition buffer size > 10MB
            if retry >= self.talosProducerConfig.get_put_message_max_failed_times() and \
                    self.partitionMessageQueue.get_cur_message_bytes() > \
                    self.talosProducerConfig.get_max_put_msg_bytes():
                self.clear_message_queue(e)

        return False

    def clear_message_queue(self, e=None):
        # clear MessageQueue, and execute onError callback
        bufferedUserMsgResult = UserMessageResult(
            self.partitionMessageQueue.get_all_message_list(),
            self.topicAndPartition.topicName(), self.partitionId)
        bufferedUserMsgResult.set_successful(False)
        bufferedUserMsgResult.set_cause(e)
        self.messageCallbackExecutors = MessageCallbackTask(bufferedUserMsgResult,
                                                            self.userMessageCallback)
        self.messageCallbackExecutors.start()
        self.logger.warn("partition: " + str(self.partitionId)
                         + " already reach clearMessageQueue limit," +
                         " clear " + str(len(bufferedUserMsgResult.get_message_list()))
                         + " messages in buffer")

    def process_buffered_message(self):
        remainingMessages = self.partitionMessageQueue.get_all_message_list()
        if len(remainingMessages) > 0:
            userMessageResult = UserMessageResult(
                remainingMessages, self.topicAndPartition.topicName, self.partitionId)
            userMessageResult.set_successful(False)
            galaxyTalosException = GalaxyTalosException()
            galaxyTalosException.details = str(self.topicAndPartition) + \
                                           "'s buffer not empty," + \
                                           " do callback when shutdown this partitionSender."
            galaxyTalosException.errMsg = str(self.topicAndPartition) + "'s buffer not empty"
            userMessageResult.set_cause(galaxyTalosException)
            self.messageCallbackExecutors = MessageCallbackTask(userMessageResult,
                                                                self.userMessageCallback)
            self.messageCallbackExecutors.start()

    def shutdown(self):
        # notify PartitionMessageQueue::getMessageList return;
        self.add_message([])
        self.update_state(TASK_STATE.CLOSE)
        self.executor.join()
        self.logger.info("PartitionSender for partition: "
                         + str(self.topicAndPartition.partitionId) + " finish stop")

    def add_message(self, userMessageList=None):
        self.partitionMessageQueue.add_message(userMessageList)
        if self.logger.isEnabledFor(logging.DEBUG):
            self.logger.debug("add " + str(len(userMessageList)) +
                              " messages to partition: " + str(self.partitionId))

    def close(self):
        self.update_state(TASK_STATE.CLOSE)

    @synchronized
    def get_cur_state(self):
        return self.curState

    @synchronized
    def update_state(self, targetState=None):
        self.logger.info("PartitionSender for Partition: " + str(self.partitionId)
                         + " update status from: " + str(self.curState) + " to: " +
                         str(targetState))
        if targetState == TASK_STATE.ACTIVE:
            self.logger.error("targetState can never be ACTIVE, updateState error for: "
                              + str(self.partitionId))
            return
        if targetState == TASK_STATE.CLOSE:
            if self.curState == TASK_STATE.ACTIVE:
                self.curState = TASK_STATE.CLOSE
            else:
                self.logger.error("targetState is CLOSE, but curState is: " +
                                  str(self.curState) + " for partition: " +
                                  str(self.partitionId))
        return

    def is_flush_finish(self):
        messageQueueSize = self.partitionMessageQueue.get_cur_message_bytes()
        if messageQueueSize > 0:
            return False
        else:
            return True




#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
#

from io import BytesIO
import gzip
import logging
import traceback


class Gzip:
    logger = logging.getLogger("Gzip")

    def __init__(self):
        pass

    def compress(self, c_data):
        buf = BytesIO()
        try:
            with gzip.GzipFile(mode='wb', fileobj=buf) as f:
                f.write(c_data)
            return buf.getvalue()
        except Exception as e:
            self.logger.error("compress wrong" + str(traceback.format_exc()))
        finally:
            f.close()

    def decompress(self, c_data):
        try:
            buf = BytesIO(c_data)
            with gzip.GzipFile(mode='rb', fileobj=buf) as f:
                return f.read()
        except Exception as e:
            self.logger.error("uncompress wrong" + str(traceback.format_exc()))
        finally:
            f.close()




#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 
 
import socket

host = 'cnbj4-talos.api.xiaomi.net'
port = 80
source_address = None
timeout = 5

res = socket.create_connection((host, port), timeout)
test = 1



import unittest

from talos.thrift.message.ttypes import MessageAndOffset
from talos.thrift.message.ttypes import Message
from talos.thrift.consumer.ttypes import UpdateOffsetResponse
from talos.thrift.consumer.ttypes import LockPartitionResponse
from talos.thrift.consumer.ttypes import QueryOffsetResponse
from talos.thrift.topic.ttypes import TopicTalosResourceName
from talos.thrift.common.ttypes import GalaxyTalosException
from talos.thrift.common.ttypes import ErrorCode
from talos.consumer.SimpleConsumer import SimpleConsumer
from talos.consumer.MessageReader import MessageReader
from talos.consumer.PartitionFetcher import PartitionFetcher
from talos.client.TalosClientFactory import ConsumerClient
from mock import Mock
import time


class test_PartitionFetcher(unittest.TestCase):
    topicName = "MyTopic"
    resourceName = "12345#MyTopic#34595fkdiso456i390"
    partitionId = 7
    consumerGroup = "MyConsumerGroup"
    workerId = "workerId"

    def setUp(self):
        messageAndOffset1 = MessageAndOffset(Message("message1"), 1)
        messageAndOffset2 = MessageAndOffset(Message("message2"), 2)
        messageAndOffset3 = MessageAndOffset(Message("message3"), 3)
        messageAndOffsetList = [messageAndOffset1, messageAndOffset2, messageAndOffset3]
        messageAndOffset4 = MessageAndOffset(Message("message4"), 4)
        messageAndOffset5 = MessageAndOffset(Message("message5"), 5)
        messageAndOffsetList2 = [messageAndOffset4, messageAndOffset5]

        self.consumerClientMock = ConsumerClient
        self.simpleConsumerMock = SimpleConsumer
        self.messageReaderMock = Mock(MessageReader)

        self.partitionFetcher = PartitionFetcher(consumerGroup=self.consumerGroup,
                                                 topicName=self.topicName,
                                                 topicTalosResourceName=TopicTalosResourceName(self.resourceName),
                                                 partitionId=self.partitionId,
                                                 workerId=self.workerId,
                                                 consumerClient=self.consumerClientMock,
                                                 simpleConsumer=self.simpleConsumerMock,
                                                 messageReader=self.messageReaderMock)

        # mock value for simpleConsumerMock and consumerClientMock
        self.simpleConsumerMock.fetch_message = Mock(side_effect=[messageAndOffsetList, messageAndOffsetList2])
        self.consumerClientMock.update_offset = Mock(return_value=UpdateOffsetResponse(True))
        self.consumerClientMock.unlock_partition = Mock(return_value=None)

        successPartitionList = [self.partitionId]
        lockPartitionResponse = LockPartitionResponse(successPartitionList)
        self.consumerClientMock.lock_partition = Mock(return_value=lockPartitionResponse)
        queryOffsetResponse = QueryOffsetResponse(0)
        self.consumerClientMock.query_offset = Mock(return_value=queryOffsetResponse)

    def tearDown(self):
        self.partitionFetcher.shutdown()

    def test_LockFailedFromLockedToUnlocked(self):
        self.consumerClientMock.lock_partition = Mock(side_effect=Exception())

        self.partitionFetcher.lock()
        # sleep to wait stealing lock failed
        time.sleep(0.04)
        self.assertEqual(False, self.partitionFetcher.is_serving())

    def test_LockFailedWhenNullSuccessList(self):
        successPartitionList = []
        lockPartitionResponse = LockPartitionResponse(successPartitionList)
        self.consumerClientMock.lock_partition = Mock(return_value=lockPartitionResponse)

        self.partitionFetcher.lock()
        # sleep to wait lock
        time.sleep(0.05)
        self.assertEqual(False, self.partitionFetcher.is_serving())

    def test_LockFailedWhenGetStartOffset(self):
        self.messageReaderMock.init_start_offset = Mock(side_effect=Exception)

        self.partitionFetcher.lock()
        time.sleep(0.05)
        self.assertEqual(False, self.partitionFetcher.is_serving())

    def test_GetMessageAndCommitOffset(self):
        self.partitionFetcher.lock()
        self.assertEqual(True, self.partitionFetcher.is_holding_lock())
        time.sleep(0.1)
        self.assertEqual(True, self.partitionFetcher.is_holding_lock())
        self.partitionFetcher.unlock()
        self.assertEqual(True, self.partitionFetcher.is_holding_lock())
        time.sleep(0.05)
        self.assertEqual(False, self.partitionFetcher.is_serving())

    def test_CallGetMessageMultipleTimesAndCommitOffset(self):
        self.partitionFetcher.lock()
        self.assertEqual(True, self.partitionFetcher.is_holding_lock())
        time.sleep(0.5)
        self.assertEqual(True, self.partitionFetcher.is_holding_lock())
        self.partitionFetcher.unlock()

    def test_PartitionNotServing(self):
        self.simpleConsumerMock.fetch_message = Mock(side_effect=GalaxyTalosException(
            errorCode=ErrorCode.PARTITION_NOT_SERVING))
        self.partitionFetcher.lock()
        time.sleep(0.5)

    def test_OffsetOutOfRange(self):
        self.simpleConsumerMock.fetch_message = Mock(side_effect=GalaxyTalosException(
            errorCode=ErrorCode.MESSAGE_OFFSET_OUT_OF_RANGE))
        self.partitionFetcher.lock()
        time.sleep(0.5)

    def test_LockLock(self):
        self.partitionFetcher.lock()
        time.sleep(0.05)
        self.partitionFetcher.lock()
        time.sleep(0.05)

    def testUnlockUnlock(self):
        self.partitionFetcher.lock()
        self.partitionFetcher.unlock()
        time.sleep(0.05)
        self.partitionFetcher.unlock()

    def test_UnlockLock(self):
        self.partitionFetcher.lock()
        self.partitionFetcher.unlock()
        self.partitionFetcher.lock()


if __name__ == '__main__':
    unittest.main()



#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 

from talos.client.serialization import MessageSerializerV1
from talos.client.serialization import MessageSerializerV2
from talos.client.serialization import MessageSerializerV3


class MessageSerializationFactory:

    _messageSerializer = {
        "V1": MessageSerializerV1.MessageSerializerV1(),
        "V2": MessageSerializerV2.MessageSerializerV2(),
        "V3": MessageSerializerV3.MessageSerializerV3()
    }

    def get_default_message_version(self):
        return 'V3'




#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 

from talos.utils import Utils
from talos.client.TalosClientConfig import TalosClientConfig
from talos.producer.TalosProducerConfigKey import TalosProducerConfigKeys


class TalosProducerConfig(TalosClientConfig):
    _service_endpoint = str
    _maxBufferedMsgNumber = int
    _maxBufferedMsgBytes = int
    _maxBufferedMsgTime = int
    _maxPutMsgNumber = int
    _maxPutMsgBytes = int

    def __init__(self, pro=None):
        self._properties = pro
        self._maxBufferedMsgNumber = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MESSAGE_NUMBER,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MESSAGE_NUMBER_DEFAULT)
        self._maxBufferedMsgBytes = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MESSAGE_BYTES,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MESSAGE_BYTES_DEFAULT)
        self._maxBufferedMsgTime = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MILLI_SECS,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MILLI_SECS_DEFAULT)
        self._maxPutMsgNumber = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER_DEFAULT)
        self._maxPutMsgBytes = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES_DEFAULT)
        self._threadPoolsize = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_THREAD_POOL_SIZE,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_THREAD_POOL_SIZE_DEFAULT)
        self._checkPartitionInterval = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_CHECK_PARTITION_INTERVAL,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_CHECK_PARTITION_INTERVAL_DEFAULT)
        self._updatePartitionIdInterval = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_UPDATE_PARTITIONID_INTERVAL,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_UPDATE_PARTITIONID_INTERVAL_DEFAULT)
        self._waitPartitionWorkingTime = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_WAIT_PARTITION_WORKING_TIME,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_WAIT_PARTITION_WORKING_TIME_DEFAULT)
        self._updatePartitionMsgNum = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_UPDATE_PARTITION_MSGNUMBER,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_UPDATE_PARTITION_MSGNUMBER_DEFAULT)
        self._compressionType = pro.get(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_COMPRESSION_TYPE,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_COMPRESSION_TYPE_DEFAULT)

        if not (self._compressionType == "NONE" or self._compressionType == "GZIP"
                or self._compressionType == "SNAPPY"):
            raise RuntimeError("Unsupported Compression Type: " + self._compressionType)

    def parameter_checking(self):
        Utils.check_parameter_range(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER,
            self._maxPutMsgNumber,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER_MINIMUM,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER_MAXIMUM)
        Utils.check_parameter_range(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES,
            self._maxPutMsgBytes,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES_MINIMUM,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES_MAXIMUM)
        Utils.check_parameter_range(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_CHECK_PARTITION_INTERVAL,
            self._checkPartitionInterval,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_CHECK_PARTITION_INTERVAL_MINIMUM,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_CHECK_PARTITION_INTERVAL_MAXIMUM)
        Utils.check_parameter_range(
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_UPDATE_PARTITIONID_INTERVAL,
            self._updatePartitionIdInterval,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_UPDATE_PARTITIONID_INTERVAL_MINIMUM,
            TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_UPDATE_PARTITIONID_INTERVAL_MAXIMUM)

    def get_max_buffered_msg_number(self):
        return self._maxBufferedMsgNumber

    def get_max_buffered_msg_bytes(self):
        return self._maxBufferedMsgBytes

    def get_max_buffered_msg_time(self):
        return self._maxBufferedMsgTime

    def get_max_put_msg_number(self):
        return self._maxPutMsgNumber

    def get_max_put_msg_bytes(self):
        return self._maxBufferedMsgBytes

    def get_thread_pool_size(self):
        return self._threadPoolsize

    def get_check_partition_interval(self):
        return self._checkPartitionInterval

    def get_update_partition_id_interval(self):
        return self._updatePartitionIdInterval

    def get_wait_partition_working_time(self):
        return self._waitPartitionWorkingTime

    def get_update_partition_msg_number(self):
        return self._updatePartitionMsgNum

    def get_compression_type(self):
        return self._compressionType

    def get_service_endpoint(self):
        return super(TalosProducerConfig, self).get_service_endpoint()




#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 

from talos.client.TopicAbnormalCallback import TopicAbnormalCallback
import logging


class SimpleAbnormalCallback(TopicAbnormalCallback):
	logger = logging.getLogger("SimpleAbnormalCallback")

	def __init__(self):
		pass

	def abnormal_handler(self, topicTalosResourceName=None, throwable=None):
		self.logger.error("Topic abnormal exception for: " + topicTalosResourceName.topicTalosResourceName + throwable.message)




#
# Autogenerated by Thrift Compiler (0.9.3)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

import talos.thrift.common.TalosBaseService
import logging
from talos.thrift.metric.ttypes import *
from talos.thrift.Thrift import TProcessor
from talos.thrift.transport import TTransport
from talos.thrift.protocol import TBinaryProtocol, TProtocol
try:
  from talos.thrift.protocol import fastbinary
except:
  fastbinary = None


class Iface(talos.thrift.common.TalosBaseService.Iface):
  def listTopics(self):
    """
    listTopicsInfo will return the topics which owned by the call developer
    and its describe information.

    """
    pass

  def queryConsumerGroup(self, request):
    """
    query ConsumerGroup


    Parameters:
     - request
    """
    pass

  def getTopicConsumeUnit(self, request):
    """
    get TopicConsumeUnit by consumerGroup and topicName


    Parameters:
     - request
    """
    pass

  def queryTopicConsumeUnit(self, request):
    """
    query TopicConsumeUnit by topicName


    Parameters:
     - request
    """
    pass


class Client(talos.thrift.common.TalosBaseService.Client, Iface):
  def __init__(self, iprot, oprot=None):
    talos.thrift.common.TalosBaseService.Client.__init__(self, iprot, oprot)

  def listTopics(self):
    """
    listTopicsInfo will return the topics which owned by the call developer
    and its describe information.

    """
    self.send_listTopics()
    return self.recv_listTopics()

  def send_listTopics(self):
    self._oprot.writeMessageBegin('listTopics', TMessageType.CALL, self._seqid)
    args = listTopics_args()
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_listTopics(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = listTopics_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "listTopics failed: unknown result")

  def queryConsumerGroup(self, request):
    """
    query ConsumerGroup


    Parameters:
     - request
    """
    self.send_queryConsumerGroup(request)
    return self.recv_queryConsumerGroup()

  def send_queryConsumerGroup(self, request):
    self._oprot.writeMessageBegin('queryConsumerGroup', TMessageType.CALL, self._seqid)
    args = queryConsumerGroup_args()
    args.request = request
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_queryConsumerGroup(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = queryConsumerGroup_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "queryConsumerGroup failed: unknown result")

  def getTopicConsumeUnit(self, request):
    """
    get TopicConsumeUnit by consumerGroup and topicName


    Parameters:
     - request
    """
    self.send_getTopicConsumeUnit(request)
    return self.recv_getTopicConsumeUnit()

  def send_getTopicConsumeUnit(self, request):
    self._oprot.writeMessageBegin('getTopicConsumeUnit', TMessageType.CALL, self._seqid)
    args = getTopicConsumeUnit_args()
    args.request = request
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_getTopicConsumeUnit(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = getTopicConsumeUnit_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "getTopicConsumeUnit failed: unknown result")

  def queryTopicConsumeUnit(self, request):
    """
    query TopicConsumeUnit by topicName


    Parameters:
     - request
    """
    self.send_queryTopicConsumeUnit(request)
    return self.recv_queryTopicConsumeUnit()

  def send_queryTopicConsumeUnit(self, request):
    self._oprot.writeMessageBegin('queryTopicConsumeUnit', TMessageType.CALL, self._seqid)
    args = queryTopicConsumeUnit_args()
    args.request = request
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_queryTopicConsumeUnit(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = queryTopicConsumeUnit_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "queryTopicConsumeUnit failed: unknown result")


class Processor(talos.thrift.common.TalosBaseService.Processor, Iface, TProcessor):
  def __init__(self, handler):
    talos.thrift.common.TalosBaseService.Processor.__init__(self, handler)
    self._processMap["listTopics"] = Processor.process_listTopics
    self._processMap["queryConsumerGroup"] = Processor.process_queryConsumerGroup
    self._processMap["getTopicConsumeUnit"] = Processor.process_getTopicConsumeUnit
    self._processMap["queryTopicConsumeUnit"] = Processor.process_queryTopicConsumeUnit

  def process(self, iprot, oprot):
    (name, type, seqid) = iprot.readMessageBegin()
    if name not in self._processMap:
      iprot.skip(TType.STRUCT)
      iprot.readMessageEnd()
      x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
      oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
      x.write(oprot)
      oprot.writeMessageEnd()
      oprot.trans.flush()
      return
    else:
      self._processMap[name](self, seqid, iprot, oprot)
    return True

  def process_listTopics(self, seqid, iprot, oprot):
    args = listTopics_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = listTopics_result()
    try:
      result.success = self._handler.listTopics()
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("listTopics", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_queryConsumerGroup(self, seqid, iprot, oprot):
    args = queryConsumerGroup_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = queryConsumerGroup_result()
    try:
      result.success = self._handler.queryConsumerGroup(args.request)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("queryConsumerGroup", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_getTopicConsumeUnit(self, seqid, iprot, oprot):
    args = getTopicConsumeUnit_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = getTopicConsumeUnit_result()
    try:
      result.success = self._handler.getTopicConsumeUnit(args.request)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("getTopicConsumeUnit", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_queryTopicConsumeUnit(self, seqid, iprot, oprot):
    args = queryTopicConsumeUnit_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = queryTopicConsumeUnit_result()
    try:
      result.success = self._handler.queryTopicConsumeUnit(args.request)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("queryTopicConsumeUnit", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()


# HELPER FUNCTIONS AND STRUCTURES

class listTopics_args:

  thrift_spec = (
  )

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('listTopics_args')
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class listTopics_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (talos.thrift.topic.ttypes.ListTopicsInfoResponse, talos.thrift.topic.ttypes.ListTopicsInfoResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = talos.thrift.topic.ttypes.ListTopicsInfoResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('listTopics_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class queryConsumerGroup_args:
  """
  Attributes:
   - request
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'request', (QueryConsumerGroupRequest, QueryConsumerGroupRequest.thrift_spec), None, ), # 1
  )

  def __init__(self, request=None,):
    self.request = request

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.request = QueryConsumerGroupRequest()
          self.request.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('queryConsumerGroup_args')
    if self.request is not None:
      oprot.writeFieldBegin('request', TType.STRUCT, 1)
      self.request.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.request)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class queryConsumerGroup_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (QueryConsumerGroupResponse, QueryConsumerGroupResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = QueryConsumerGroupResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('queryConsumerGroup_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class getTopicConsumeUnit_args:
  """
  Attributes:
   - request
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'request', (GetTopicConsumeUnitRequest, GetTopicConsumeUnitRequest.thrift_spec), None, ), # 1
  )

  def __init__(self, request=None,):
    self.request = request

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.request = GetTopicConsumeUnitRequest()
          self.request.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('getTopicConsumeUnit_args')
    if self.request is not None:
      oprot.writeFieldBegin('request', TType.STRUCT, 1)
      self.request.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.request)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class getTopicConsumeUnit_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (GetTopicConsumeUnitResponse, GetTopicConsumeUnitResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = GetTopicConsumeUnitResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('getTopicConsumeUnit_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class queryTopicConsumeUnit_args:
  """
  Attributes:
   - request
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'request', (QueryTopicConsumeUnitRequest, QueryTopicConsumeUnitRequest.thrift_spec), None, ), # 1
  )

  def __init__(self, request=None,):
    self.request = request

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.request = QueryTopicConsumeUnitRequest()
          self.request.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('queryTopicConsumeUnit_args')
    if self.request is not None:
      oprot.writeFieldBegin('request', TType.STRUCT, 1)
      self.request.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.request)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class queryTopicConsumeUnit_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (QueryTopicConsumeUnitResponse, QueryTopicConsumeUnitResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = QueryTopicConsumeUnitResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('queryTopicConsumeUnit_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 
 
from talos.thrift.common.ttypes import GalaxyTalosException


class ProducerNotActiveException(GalaxyTalosException):
    def __init__(self, detailMsg=None):
        GalaxyTalosException().__init__(errMsg=detailMsg)
        self.errMsg = detailMsg








from setuptools import find_packages, setup

setup(
    name='talos-python3',
    version='0.0.6',
    author='huyumei',
    author_email='huyumei@xiaomi.com',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'cffi',
        'python-snappy',
        'atomic'
    ],
)




import unittest
from talos.producer.UserMessageCallback import UserMessageCallback
from talos.producer.TalosProducerConfigKey import TalosProducerConfigKeys
from talos.producer.UserMessage import UserMessage
from talos.producer.TalosProducer import TalosProducer
from talos.producer.PartitionSender import PartitionSender
from talos.client.TalosClientConfigkeys import TalosClientConfigKeys
from talos.client.TalosClientFactory import MessageClient
from talos.client.TalosClientConfig import TalosClientConfig
from talos.thrift.message.ttypes import Message
from talos.thrift.message.ttypes import MessageType
from talos.thrift.topic.ttypes import TopicTalosResourceName
from talos.thrift.message.ttypes import PutMessageResponse
from talos.thrift.common.ttypes import GalaxyTalosException
from mock import Mock
from atomic import AtomicLong
from talos.utils import Utils
import threading
import logging

msgPutSuccessCount = 0
msgPutFailureCount = 0


def clearCounter():
	global msgPutFailureCount
	msgPutFailureCount = 0
	global msgPutSuccessCount
	msgPutSuccessCount = 0


class MessageCallback(UserMessageCallback):

	def on_success(self, userMessageResult=None):
		global msgPutSuccessCount
		test = msgPutSuccessCount
		msgPutSuccessCount += len(userMessageResult.get_message_list())

	def on_error(self, userMessageResult=None):
		global msgPutFailureCount
		msgPutFailureCount += len(userMessageResult.get_message_list())


class test_PartitionSender(unittest.TestCase):
	logging.basicConfig()

	producerMaxBufferedMillSecs = 0.01
	producerMaxPutMsgNumber = 5
	producerMaxPutMsgBytes = 50
	producerMaxBufferedMsgNum = 50

	resourceName0 = "12345#TopicName#july7777770009990"
	resourceName1 = "12345#TopicName#july7777770009991"
	resourceName2 = "12345#TopicName#july7777770009992"
	resourceName3 = "12345#TopicName#july7777770009993"

	topicName = "TopicName"
	partitionId = 0
	requestId = AtomicLong(1)
	globalLock = threading.Condition()

	def setUp(self):
		properties = dict()
		properties[TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MILLI_SECS] \
			= self.producerMaxBufferedMillSecs
		properties[TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_NUMBER] \
			= self.producerMaxPutMsgNumber
		properties[TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_PUT_MESSAGE_BYTES] \
			= self.producerMaxPutMsgBytes
		properties[TalosProducerConfigKeys.GALAXY_TALOS_PRODUCER_MAX_BUFFERED_MESSAGE_NUMBER] \
			= self.producerMaxBufferedMsgNum
		properties[TalosClientConfigKeys.GALAXY_TALOS_SERVICE_ENDPOINT] = "testURI"
		self.talosProducerConfig = TalosClientConfig(properties)

		self.talosResourceName0 = TopicTalosResourceName(self.resourceName0)
		self.talosResourceName1 = TopicTalosResourceName(self.resourceName1)
		self.talosResourceName2 = TopicTalosResourceName(self.resourceName2)
		self.talosResourceName3 = TopicTalosResourceName(self.resourceName3)
		self.messageCallback = MessageCallback()
		self.messageCallbackExecutors = threading.Thread()
		self.messageClientMock = Mock(MessageClient)
		self.producerMock = Mock(TalosProducer)

		self.userMessageList = []
		timestamp = Utils.current_time_mills()
		self.userMessage1 = UserMessage(Message(message="hello", createTimestamp=timestamp, messageType=MessageType.BINARY))
		self.userMessage2 = UserMessage(Message(message="world", createTimestamp=timestamp, messageType=MessageType.BINARY))
		self.userMessage3 = UserMessage(Message(message="nice day", createTimestamp=timestamp, messageType=MessageType.BINARY))
		self.userMessage4 = UserMessage(Message(message="good guy", createTimestamp=timestamp, messageType=MessageType.BINARY))

	def tearDown(self):
		pass

	def test_partition_sender_success_put(self):
		self.userMessageList = [self.userMessage1, self.userMessage2, self.userMessage3, self.userMessage4]

		self.messageClientMock.put_message = Mock(return_value=PutMessageResponse())
		# self.producerMock.is_active = Mock(side_effect=[True, True,True, True, True, False])
		self.producerMock.is_active = Mock(return_value=True)
		self.producerMock.is_disabled = Mock(return_value=False)

		partitionSender = PartitionSender(partitionId=self.partitionId,
										  topicName=self.topicName,
										  topicTalosResourceName=self.talosResourceName1,
										  requestId=self.requestId,
										  clientId=Utils.generate_client_id(clientIp=self.talosProducerConfig.get_client_ip(), prefix=""),
										  talosProducerConfig=self.talosProducerConfig,
										  messageClient=self.messageClientMock,
										  userMessageCallback=self.messageCallback,
										  messageCallbackExecutors=self.messageCallbackExecutors,
										  globalLock=self.globalLock,
										  producer=self.producerMock)
		addCount = 1
		while addCount > 0:
			partitionSender.add_message(self.userMessageList)
			addCount -= 1

		partitionSender.shutdown()

		global msgPutSuccessCount
		self.assertEqual(4, msgPutSuccessCount)
		clearCounter()

	def test_partition_sender_failed_put(self):
		self.userMessageList = [self.userMessage1, self.userMessage2,
								self.userMessage3, self.userMessage4]

		self.messageClientMock.put_message = Mock(side_effect=GalaxyTalosException(errMsg="put failed"))
		self.producerMock.is_active = Mock(side_effect=[True, False])

		partitionSender = PartitionSender(partitionId=self.partitionId,
										  topicName=self.topicName,
										  topicTalosResourceName=self.talosResourceName1,
										  requestId=self.requestId,
										  clientId=Utils.generate_client_id(clientIp=self.talosProducerConfig.get_client_ip(), prefix=""),
										  talosProducerConfig=self.talosProducerConfig,
										  messageClient=self.messageClientMock,
										  userMessageCallback=self.messageCallback,
										  messageCallbackExecutors=self.messageCallbackExecutors,
										  globalLock=self.globalLock,
										  producer=self.producerMock)
		partitionSender.add_message(self.userMessageList)

		partitionSender.shutdown()
		self.assertEquals(0, msgPutSuccessCount)
		self.assertEquals(4, msgPutFailureCount)
		clearCounter()

	def testPartitionQueueWaitToPut(self):
		pass

	def testPartitiionNotServingDelay(self):
		pass


if __name__ == '__main__':
	unittest.main()



#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 


class MessageVersion(object):

    _version = int

    def __init__(self, version=None):
        self._version = version

    def get_version_number(self):
        return self._version

    def get_version(self):
        if self._version == 1:
            return "V1"
        elif self._version == 2:
            return "V2"
        elif self._version == 3:
            return "V3"
        return "UNKNOW"








#
# Copyright 2020, Xiaomi.
# All rights reserved.
# Author: huyumei@xiaomi.com
# 

from talos.thrift.message.ttypes import Message
from talos.utils import Utils


class UserMessage:
    message = Message
    timestamp = int
    messageSize = int

    def __init__(self, message=None):
        self.message = message
        self.timestamp = Utils.current_time_mills()
        self.messageSize = len(message.message)
        if message.sequenceNumber:
            self.messageSize += len(message.sequenceNumber)

    def get_message(self):
        return self.message

    def get_timestamp(self):
        return self.timestamp

    def get_message_size(self):
        return self.messageSize



#
# Autogenerated by Thrift Compiler (0.9.3)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

import talos.thrift.common.TalosBaseService
import logging
from talos.thrift.quota.ttypes import *
from talos.thrift.Thrift import TProcessor
from talos.thrift.transport import TTransport
from talos.thrift.protocol import TBinaryProtocol, TProtocol
try:
  from talos.thrift.protocol import fastbinary
except:
  fastbinary = None


class Iface(talos.thrift.common.TalosBaseService.Iface):
  def setUserQuota(self, request):
    """
    setQuota, the caller must be talos admin;
    Note: the developer can not create topic until Talos admin setUserQuota


    Parameters:
     - request
    """
    pass

  def listUserQuota(self):
    """
    listQuota, the caller must be talos admin;

    """
    pass

  def queryUserQuota(self):
    """
    queryQuota, this will return the quota with the caller;
    the caller not need to be talos admin

    """
    pass

  def deleteUserQuota(self, request):
    """
    deleteQuota, the caller must be talos admin;


    Parameters:
     - request
    """
    pass

  def applyQuota(self, request):
    """
    Parameters:
     - request
    """
    pass

  def autoApplyQuota(self, request):
    """
    Parameters:
     - request
    """
    pass

  def listQuota(self):
    pass

  def listPendingQuota(self):
    pass

  def approveQuota(self, request):
    """
    Parameters:
     - request
    """
    pass

  def revokeQuota(self, request):
    """
    Parameters:
     - request
    """
    pass


class Client(talos.thrift.common.TalosBaseService.Client, Iface):
  def __init__(self, iprot, oprot=None):
    talos.thrift.common.TalosBaseService.Client.__init__(self, iprot, oprot)

  def setUserQuota(self, request):
    """
    setQuota, the caller must be talos admin;
    Note: the developer can not create topic until Talos admin setUserQuota


    Parameters:
     - request
    """
    self.send_setUserQuota(request)
    self.recv_setUserQuota()

  def send_setUserQuota(self, request):
    self._oprot.writeMessageBegin('setUserQuota', TMessageType.CALL, self._seqid)
    args = setUserQuota_args()
    args.request = request
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_setUserQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = setUserQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.e is not None:
      raise result.e
    return

  def listUserQuota(self):
    """
    listQuota, the caller must be talos admin;

    """
    self.send_listUserQuota()
    return self.recv_listUserQuota()

  def send_listUserQuota(self):
    self._oprot.writeMessageBegin('listUserQuota', TMessageType.CALL, self._seqid)
    args = listUserQuota_args()
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_listUserQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = listUserQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "listUserQuota failed: unknown result")

  def queryUserQuota(self):
    """
    queryQuota, this will return the quota with the caller;
    the caller not need to be talos admin

    """
    self.send_queryUserQuota()
    return self.recv_queryUserQuota()

  def send_queryUserQuota(self):
    self._oprot.writeMessageBegin('queryUserQuota', TMessageType.CALL, self._seqid)
    args = queryUserQuota_args()
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_queryUserQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = queryUserQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "queryUserQuota failed: unknown result")

  def deleteUserQuota(self, request):
    """
    deleteQuota, the caller must be talos admin;


    Parameters:
     - request
    """
    self.send_deleteUserQuota(request)
    self.recv_deleteUserQuota()

  def send_deleteUserQuota(self, request):
    self._oprot.writeMessageBegin('deleteUserQuota', TMessageType.CALL, self._seqid)
    args = deleteUserQuota_args()
    args.request = request
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_deleteUserQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = deleteUserQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.e is not None:
      raise result.e
    return

  def applyQuota(self, request):
    """
    Parameters:
     - request
    """
    self.send_applyQuota(request)
    self.recv_applyQuota()

  def send_applyQuota(self, request):
    self._oprot.writeMessageBegin('applyQuota', TMessageType.CALL, self._seqid)
    args = applyQuota_args()
    args.request = request
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_applyQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = applyQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.e is not None:
      raise result.e
    return

  def autoApplyQuota(self, request):
    """
    Parameters:
     - request
    """
    self.send_autoApplyQuota(request)
    self.recv_autoApplyQuota()

  def send_autoApplyQuota(self, request):
    self._oprot.writeMessageBegin('autoApplyQuota', TMessageType.CALL, self._seqid)
    args = autoApplyQuota_args()
    args.request = request
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_autoApplyQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = autoApplyQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.e is not None:
      raise result.e
    return

  def listQuota(self):
    self.send_listQuota()
    return self.recv_listQuota()

  def send_listQuota(self):
    self._oprot.writeMessageBegin('listQuota', TMessageType.CALL, self._seqid)
    args = listQuota_args()
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_listQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = listQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "listQuota failed: unknown result")

  def listPendingQuota(self):
    self.send_listPendingQuota()
    return self.recv_listPendingQuota()

  def send_listPendingQuota(self):
    self._oprot.writeMessageBegin('listPendingQuota', TMessageType.CALL, self._seqid)
    args = listPendingQuota_args()
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_listPendingQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = listPendingQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "listPendingQuota failed: unknown result")

  def approveQuota(self, request):
    """
    Parameters:
     - request
    """
    self.send_approveQuota(request)
    return self.recv_approveQuota()

  def send_approveQuota(self, request):
    self._oprot.writeMessageBegin('approveQuota', TMessageType.CALL, self._seqid)
    args = approveQuota_args()
    args.request = request
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_approveQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = approveQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "approveQuota failed: unknown result")

  def revokeQuota(self, request):
    """
    Parameters:
     - request
    """
    self.send_revokeQuota(request)
    return self.recv_revokeQuota()

  def send_revokeQuota(self, request):
    self._oprot.writeMessageBegin('revokeQuota', TMessageType.CALL, self._seqid)
    args = revokeQuota_args()
    args.request = request
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

  def recv_revokeQuota(self):
    iprot = self._iprot
    (fname, mtype, rseqid) = iprot.readMessageBegin()
    if mtype == TMessageType.EXCEPTION:
      x = TApplicationException()
      x.read(iprot)
      iprot.readMessageEnd()
      raise x
    result = revokeQuota_result()
    result.read(iprot)
    iprot.readMessageEnd()
    if result.success is not None:
      return result.success
    if result.e is not None:
      raise result.e
    raise TApplicationException(TApplicationException.MISSING_RESULT, "revokeQuota failed: unknown result")


class Processor(talos.thrift.common.TalosBaseService.Processor, Iface, TProcessor):
  def __init__(self, handler):
    talos.thrift.common.TalosBaseService.Processor.__init__(self, handler)
    self._processMap["setUserQuota"] = Processor.process_setUserQuota
    self._processMap["listUserQuota"] = Processor.process_listUserQuota
    self._processMap["queryUserQuota"] = Processor.process_queryUserQuota
    self._processMap["deleteUserQuota"] = Processor.process_deleteUserQuota
    self._processMap["applyQuota"] = Processor.process_applyQuota
    self._processMap["autoApplyQuota"] = Processor.process_autoApplyQuota
    self._processMap["listQuota"] = Processor.process_listQuota
    self._processMap["listPendingQuota"] = Processor.process_listPendingQuota
    self._processMap["approveQuota"] = Processor.process_approveQuota
    self._processMap["revokeQuota"] = Processor.process_revokeQuota

  def process(self, iprot, oprot):
    (name, type, seqid) = iprot.readMessageBegin()
    if name not in self._processMap:
      iprot.skip(TType.STRUCT)
      iprot.readMessageEnd()
      x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
      oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
      x.write(oprot)
      oprot.writeMessageEnd()
      oprot.trans.flush()
      return
    else:
      self._processMap[name](self, seqid, iprot, oprot)
    return True

  def process_setUserQuota(self, seqid, iprot, oprot):
    args = setUserQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = setUserQuota_result()
    try:
      self._handler.setUserQuota(args.request)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("setUserQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_listUserQuota(self, seqid, iprot, oprot):
    args = listUserQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = listUserQuota_result()
    try:
      result.success = self._handler.listUserQuota()
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("listUserQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_queryUserQuota(self, seqid, iprot, oprot):
    args = queryUserQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = queryUserQuota_result()
    try:
      result.success = self._handler.queryUserQuota()
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("queryUserQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_deleteUserQuota(self, seqid, iprot, oprot):
    args = deleteUserQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = deleteUserQuota_result()
    try:
      self._handler.deleteUserQuota(args.request)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("deleteUserQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_applyQuota(self, seqid, iprot, oprot):
    args = applyQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = applyQuota_result()
    try:
      self._handler.applyQuota(args.request)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("applyQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_autoApplyQuota(self, seqid, iprot, oprot):
    args = autoApplyQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = autoApplyQuota_result()
    try:
      self._handler.autoApplyQuota(args.request)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("autoApplyQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_listQuota(self, seqid, iprot, oprot):
    args = listQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = listQuota_result()
    try:
      result.success = self._handler.listQuota()
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("listQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_listPendingQuota(self, seqid, iprot, oprot):
    args = listPendingQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = listPendingQuota_result()
    try:
      result.success = self._handler.listPendingQuota()
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("listPendingQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_approveQuota(self, seqid, iprot, oprot):
    args = approveQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = approveQuota_result()
    try:
      result.success = self._handler.approveQuota(args.request)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("approveQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()

  def process_revokeQuota(self, seqid, iprot, oprot):
    args = revokeQuota_args()
    args.read(iprot)
    iprot.readMessageEnd()
    result = revokeQuota_result()
    try:
      result.success = self._handler.revokeQuota(args.request)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except talos.thrift.common.ttypes.GalaxyTalosException as e:
      msg_type = TMessageType.REPLY
      result.e = e
    except Exception as ex:
      msg_type = TMessageType.EXCEPTION
      logging.exception(ex)
      result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
    oprot.writeMessageBegin("revokeQuota", msg_type, seqid)
    result.write(oprot)
    oprot.writeMessageEnd()
    oprot.trans.flush()


# HELPER FUNCTIONS AND STRUCTURES

class setUserQuota_args:
  """
  Attributes:
   - request
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'request', (SetUserQuotaRequest, SetUserQuotaRequest.thrift_spec), None, ), # 1
  )

  def __init__(self, request=None,):
    self.request = request

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.request = SetUserQuotaRequest()
          self.request.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('setUserQuota_args')
    if self.request is not None:
      oprot.writeFieldBegin('request', TType.STRUCT, 1)
      self.request.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.request)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class setUserQuota_result:
  """
  Attributes:
   - e
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, e=None,):
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('setUserQuota_result')
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class listUserQuota_args:

  thrift_spec = (
  )

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('listUserQuota_args')
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class listUserQuota_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (ListUserQuotaResponse, ListUserQuotaResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = ListUserQuotaResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('listUserQuota_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class queryUserQuota_args:

  thrift_spec = (
  )

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('queryUserQuota_args')
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class queryUserQuota_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (QueryUserQuotaResponse, QueryUserQuotaResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = QueryUserQuotaResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('queryUserQuota_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class deleteUserQuota_args:
  """
  Attributes:
   - request
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'request', (DeleteUserQuotaRequest, DeleteUserQuotaRequest.thrift_spec), None, ), # 1
  )

  def __init__(self, request=None,):
    self.request = request

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.request = DeleteUserQuotaRequest()
          self.request.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('deleteUserQuota_args')
    if self.request is not None:
      oprot.writeFieldBegin('request', TType.STRUCT, 1)
      self.request.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.request)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class deleteUserQuota_result:
  """
  Attributes:
   - e
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, e=None,):
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('deleteUserQuota_result')
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class applyQuota_args:
  """
  Attributes:
   - request
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'request', (ApplyQuotaRequest, ApplyQuotaRequest.thrift_spec), None, ), # 1
  )

  def __init__(self, request=None,):
    self.request = request

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.request = ApplyQuotaRequest()
          self.request.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('applyQuota_args')
    if self.request is not None:
      oprot.writeFieldBegin('request', TType.STRUCT, 1)
      self.request.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.request)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class applyQuota_result:
  """
  Attributes:
   - e
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, e=None,):
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('applyQuota_result')
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class autoApplyQuota_args:
  """
  Attributes:
   - request
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'request', (AutoApplyQuotaRequest, AutoApplyQuotaRequest.thrift_spec), None, ), # 1
  )

  def __init__(self, request=None,):
    self.request = request

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.request = AutoApplyQuotaRequest()
          self.request.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('autoApplyQuota_args')
    if self.request is not None:
      oprot.writeFieldBegin('request', TType.STRUCT, 1)
      self.request.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.request)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class autoApplyQuota_result:
  """
  Attributes:
   - e
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, e=None,):
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('autoApplyQuota_result')
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class listQuota_args:

  thrift_spec = (
  )

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('listQuota_args')
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class listQuota_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (ListQuotaResponse, ListQuotaResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = ListQuotaResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('listQuota_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class listPendingQuota_args:

  thrift_spec = (
  )

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('listPendingQuota_args')
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class listPendingQuota_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (ListPendingQuotaResponse, ListPendingQuotaResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = ListPendingQuotaResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('listPendingQuota_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class approveQuota_args:
  """
  Attributes:
   - request
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'request', (ApproveQuotaRequest, ApproveQuotaRequest.thrift_spec), None, ), # 1
  )

  def __init__(self, request=None,):
    self.request = request

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.request = ApproveQuotaRequest()
          self.request.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('approveQuota_args')
    if self.request is not None:
      oprot.writeFieldBegin('request', TType.STRUCT, 1)
      self.request.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.request)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class approveQuota_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (ApproveQuotaResponse, ApproveQuotaResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = ApproveQuotaResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('approveQuota_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class revokeQuota_args:
  """
  Attributes:
   - request
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'request', (RevokeQuotaRequest, RevokeQuotaRequest.thrift_spec), None, ), # 1
  )

  def __init__(self, request=None,):
    self.request = request

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.request = RevokeQuotaRequest()
          self.request.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('revokeQuota_args')
    if self.request is not None:
      oprot.writeFieldBegin('request', TType.STRUCT, 1)
      self.request.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.request)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class revokeQuota_result:
  """
  Attributes:
   - success
   - e
  """

  thrift_spec = (
    (0, TType.STRUCT, 'success', (RevokeQuotaResponse, RevokeQuotaResponse.thrift_spec), None, ), # 0
    (1, TType.STRUCT, 'e', (talos.thrift.common.ttypes.GalaxyTalosException, talos.thrift.common.ttypes.GalaxyTalosException.thrift_spec), None, ), # 1
  )

  def __init__(self, success=None, e=None,):
    self.success = success
    self.e = e

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 0:
        if ftype == TType.STRUCT:
          self.success = RevokeQuotaResponse()
          self.success.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 1:
        if ftype == TType.STRUCT:
          self.e = talos.thrift.common.ttypes.GalaxyTalosException()
          self.e.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('revokeQuota_result')
    if self.success is not None:
      oprot.writeFieldBegin('success', TType.STRUCT, 0)
      self.success.write(oprot)
      oprot.writeFieldEnd()
    if self.e is not None:
      oprot.writeFieldBegin('e', TType.STRUCT, 1)
      self.e.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.success)
    value = (value * 31) ^ hash(self.e)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.items()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)
